@@ -1,7 +1,7 @@
 ##
 #   Project: gespeaker - A GTK frontend for espeak  
 #    Author: Fabio Castelli <muflone@vbsimple.net>
-# Copyright: 2009 Fabio Castelli
+# Copyright: 2009-2013 Fabio Castelli
 #   License: GPL-2+
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the Free
@@ -23,8 +23,6 @@ import pygtk
 import gobject
 pygtk.require('2.0')
 
-# No more used
-# import TempfileWrapper
 import tempfile
 import os
 from gettext import gettext as _
@@ -37,8 +35,11 @@ from EspeakFrontend import EspeakFrontend
 import PreferencesWindow
 import Settings
 import handlepaths
+import plugins
 from pygtkutils import *
 
+COL_LANGUAGE, COL_NAME, COL_MBROLA = range(3)
+
 class gespeakerUI(object):
   def __init__(self):
     print 'starting %s' % handlepaths.APP_NAME
@@ -47,10 +48,32 @@ class gespeakerUI(object):
     self.tempFilename = tempfile.mkstemp(prefix=handlepaths.APP_NAME)[1]
     self.timeoutCheck = None
     self.recordToFile = None
+    self.recordToFileRequested = False
     self.variants = ((), ())
     gladeUI = handlepaths.getPath('ui', 'gespeaker.glade')
     print 'loading interface from %s' % gladeUI
     self.gladeFile = gtk.glade.XML(fname=gladeUI, domain=handlepaths.APP_NAME)
+    # Proxy maps
+    self.proxy = {
+      'text.set': self.set_text,
+      'text.cut': self.on_imgmenuEditCut_activate,
+      'text.copy': self.on_imgmenuEditCopy_activate,
+      'text.paste': self.on_imgmenuEditPaste_activate,
+      'ui.new':  self.on_imgmenuFileNew_activate,
+      'ui.open': self.on_imgmenuFileOpen_activate,
+      'ui.save': self.on_imgmenuFileSaveAs_activate,
+      'ui.record': self.on_imgmenuFileRec_activate,
+      'ui.unrecord': self.disable_record,
+      'ui.reset': self.on_imgmenuEditResetSettings_activate,
+      'ui.quit': self.on_imgmenuFileQuit_activate,
+      'ui.set_size': self.set_size,
+      'ui.set_position': self.set_position,
+      'ui.set_window': self.set_window,
+      'espeak.play': self.on_imgmenuEditPlay_activate,
+      'espeak.stop': self.on_imgmenuEditStop_activate,
+      'espeak.pause': self.on_imgmenuEditPause_activate,
+      'espeak.is_playing': self.checkIfPlaying,
+    }
     # Signals handler
     signals = {
       'on_imgmenuFileQuit_activate': self.on_imgmenuFileQuit_activate,
@@ -63,30 +86,28 @@ class gespeakerUI(object):
       'on_btnPause_toggled': self.on_btnPause_toggled,
       'on_tlbRecord_toggled': self.on_tlbRecord_toggled,
       'on_imgmenuEditPause_activate': self.on_imgmenuEditPause_activate,
-      'on_imgmenuEditRec_activate': self.on_imgmenuEditRec_activate,
+      'on_imgmenuFileRec_activate': self.on_imgmenuFileRec_activate,
       'on_imgmenuEditResetSettings_activate': self.on_imgmenuEditResetSettings_activate,
+      'on_imgmenuEditCut_activate': self.on_imgmenuEditCut_activate,
+      'on_imgmenuEditCopy_activate': self.on_imgmenuEditCopy_activate,
+      'on_imgmenuEditPaste_activate': self.on_imgmenuEditPaste_activate,
       'on_imgmenuHelpAbout_activate': self.on_imgmenuHelpAbout_activate,
       'on_imgmenuEditPreferences_activate': self.on_imgmenuEditPreferences_activate,
       'on_radioVoice_toggled': self.on_radioVoice_toggled,
       'on_cboLanguages_changed': self.on_cboLanguages_changed
     }
     self.gladeFile.signal_autoconnect(signals)
-    # Load user settings
-    Settings.load()
+    # Set clipboard
+    display = gtk.gdk.display_manager_get().get_default_display()
+    self.clipboard = gtk.Clipboard(display, "CLIPBOARD")
+
     # Load window and controls
     self.loadControls()
     self.loadSettings(True)
-    self.winMain.show()
-    
-    # Play welcome message if PlayWelcomeText is set
-    if Settings.get('PlayWelcomeText'):
-      if Settings.get('UseCustomWelcome'):
-        # Play customized welcome message
-        self.txvBuffer.set_text(Settings.get('WelcomeText'))
-      else:
-        # Play default welcome message
-        self.txvBuffer.set_text(Settings.default('WelcomeText'))
-      self.btnPlayStop.set_active(True)
+  
+  def run(self):
+    "Start main loop"
+    self.proxy['ui.set_window']('show')
     gtk.main()
   
   def loadControls(self):
@@ -114,7 +135,7 @@ class gespeakerUI(object):
     self.cboVariants = gw('cboVariants')
     self.imgmenuEditPlay = gw('imgmenuEditPlay')
     self.imgmenuEditStop = gw('imgmenuEditStop')
-    self.imgmenuEditRec = gw('imgmenuEditRec')
+    self.imgmenuFileRec = gw('imgmenuFileRec')
     self.tlbStop = gw('tlbStop')
     self.btnPlayStop = gw('btnPlayStop')
     self.btnPause = gw('btnPause')
@@ -145,7 +166,7 @@ class gespeakerUI(object):
     if defaultLanguage == 'default language':
       defaultLanguage = 'default'
     self.defaultLanguageIndex = TreeModel_find_text(
-      self.listLanguages, 0, defaultLanguage)
+      self.listLanguages, COL_LANGUAGE, defaultLanguage)
     # Prepare sorted model for voice variants
     self.listVariants = gtk.ListStore(str, str)
     self.listVariants.set_sort_column_id(0, gtk.SORT_ASCENDING)
@@ -153,17 +174,10 @@ class gespeakerUI(object):
     cell = gtk.CellRendererText()
     self.cboVariants.pack_start(cell, True)
     self.cboVariants.add_attribute(cell, 'text', 1)
-    # Restore window size
-    if Settings.get('SaveWindowSize'):
-      self.winMain.set_default_size(
-        Settings.get('MainWindowWidth'),
-        Settings.get('MainWindowHeight')
-      )
-    else:
-      self.winMain.set_default_size(
-        Settings.default('MainWindowWidth'),
-        Settings.default('MainWindowHeight')
-      )
+    self.winMain.set_default_size(
+      Settings.default('MainWindowWidth'),
+      Settings.default('MainWindowHeight')
+    )
     self.expSettings.set_expanded(Settings.get('SettingsExpander'))
 
   def loadSettings(self, loadEverything):
@@ -179,10 +193,10 @@ class gespeakerUI(object):
         self.radioVoiceFemale.set_active(True)
       # Sets default language
       language = Settings.get('VoiceLanguage')
-      languageIndex = TreeModel_find_text(self.listLanguages, 0, language)
+      languageIndex = TreeModel_find_text(self.listLanguages, COL_LANGUAGE, language)
       if not ComboBox_set_item_from_text(self.cboLanguages, 0, language):
         self.cboLanguages.set_active(self.defaultLanguageIndex)
-      Settings.set('VoiceLanguage', self.listLanguages[self.cboLanguages.get_active()][1])
+      Settings.set('VoiceLanguage', self.listLanguages[self.cboLanguages.get_active()][COL_NAME])
     # Load standard settings
     self.txvText.set_wrap_mode(
       Settings.get('WordWrap') and gtk.WRAP_WORD or gtk.WRAP_NONE)
@@ -203,29 +217,9 @@ class gespeakerUI(object):
 
   def on_imgmenuFileQuit_activate(self, widget, data=None):
     "Close the program"
-    print 'quitting'
+    plugins.signal_proxy('on_closing')
     if self.tempFilename and os.path.exists(self.tempFilename):
       os.remove(self.tempFilename)
-    # Save window size if SaveWindowSize is set
-    if Settings.get('SaveWindowSize'):
-      sizes = self.winMain.get_size()
-      Settings.set('MainWindowWidth', sizes[0])
-      Settings.set('MainWindowHeight', sizes[1])
-      Settings.set('SettingsExpander', self.expSettings.get_expanded())
-    # Save voice settings if SaveVoiceSettings is set
-    if Settings.get('SaveVoiceSettings'):
-      Settings.set('VoiceVolume', int(self.hscVolume.get_value()))
-      Settings.set('VoicePitch', int(self.hscPitch.get_value()))
-      Settings.set('VoiceSpeed', int(self.hscSpeed.get_value()))
-      Settings.set('VoiceDelay', int(self.hscDelay.get_value()))
-      Settings.set('VoiceTypeMale', self.radioVoiceMale.get_active())
-      # Save language only if different from defaultLanguageIndex
-      language = ComboBox_get_text(self.cboLanguages, 0)
-      language = self.listLanguages[self.cboLanguages.get_active()][0]
-      Settings.set('VoiceLanguage', language)
-    # Save settings
-    print 'saving settings'
-    Settings.save(clearDefaults=True)
     gtk.main_quit()
     return 0
   
@@ -233,33 +227,42 @@ class gespeakerUI(object):
     "Press button to start play, indirect cause button style"
     self.btnPlayStop.set_active(True)
   
-  def on_imgmenuFileNew_activate(self, widget, data=None):
+  def on_imgmenuFileNew_activate(self, widget, confirm=True):
     "Clears the whole text"
     if TextBuffer_get_text(self.txvBuffer):
-      dialog = DialogYesNo(
-        message=_('Do you want to delete the current text?'), 
-        default_button=gtk.RESPONSE_NO
-      )
-      dialog.set_icon_from_file(handlepaths.get_app_logo())
-      dialog.show()
-      if dialog.responseIsYes():
+      if confirm:
+        dialog = DialogYesNo(
+          message=_('Do you want to delete the current text?'), 
+          default_button=gtk.RESPONSE_NO
+        )
+        dialog.set_icon_from_file(handlepaths.get_app_logo())
+        dialog.show()
+      # Clear if confirm was not requested or if the user gave response
+      if not confirm or dialog.responseIsYes():
         self.txvBuffer.set_text('')
         print 'text cleared'
   
-  def on_imgmenuFileOpen_activate(self, widget, data=None):
+  def on_imgmenuFileOpen_activate(self, widget, filename=None):
     "Loads an external file"
-    dialog = DialogFileOpen(
-      title=_('Please select the text file to open'),
-      initialDir=os.path.expanduser('~'))
-    dialog.set_icon_from_file(handlepaths.get_app_logo())
-    dialog.addFilter(_('Text files (*.txt)'), ['*.txt'], None)
-    dialog.addFilter(_('All files'), ['*'], None)
-    if dialog.show():
+    if not filename:
+      # If filename was not provided show dialog
+      dialog = DialogFileOpen(
+        title=_('Please select the text file to open'),
+        initialDir=os.path.expanduser('~'))
+      dialog.set_icon_from_file(handlepaths.get_app_logo())
+      dialog.addFilter(_('Text files (*.txt)'), ['*.txt'], None)
+      dialog.addFilter(_('All files'), ['*'], None)
+      if dialog.show():
+        filename = dialog.filename
+      dialog.destroy()
+
+    # Open selected filename
+    if filename:
       file = None
       try:
-        file = open(dialog.filename, 'r')
+        file = open(filename, 'r')
         self.txvBuffer.set_text(file.read())
-        print 'loading text from %s' % dialog.filename
+        print 'loading text from %s' % filename
       except IOError, (errno, strerror):
         ShowDialogError(
           text=_('Error opening the file') + '\n\n%s' % strerror,
@@ -267,7 +270,7 @@ class gespeakerUI(object):
           icon=handlepaths.get_app_logo()
         )
         print 'unable to load %s (I/O error %s: %s)' % (
-          dialog.filename, errno, strerror
+          filename, errno, strerror
         )
       except:
         ShowDialogError(
@@ -275,23 +278,27 @@ class gespeakerUI(object):
           showOk=True,
           icon=handlepaths.get_app_logo()
         )
-        print 'error loading %s' % dialog.filename
+        print 'error loading %s' % filename
       if file:
         file.close()
   
-  def on_imgmenuFileSaveAs_activate(self, widget, data=None):
+  def on_imgmenuFileSaveAs_activate(self, widget, filename=None):
     "Saves the whole text in the specified filename"
-    dialog = DialogFileSave(
-      title=_('Please select where to save the text file'),
-      initialDir=os.path.expanduser('~'))
-    txtFilter = _('Text files (*.txt)')
-    dialog.addFilter(txtFilter, ['*.txt'], None)
-    dialog.addFilter(_('All files'), ['*'], None)
-    dialog.set_icon_from_file(handlepaths.get_app_logo())
-    if dialog.show():
-      filename = dialog.filename
-      if dialog.lastFilter.get_name() == txtFilter and filename[-4:] != '.txt':
-        filename += '.txt'
+    if not filename:
+      dialog = DialogFileSave(
+        title=_('Please select where to save the text file'),
+        initialDir=os.path.expanduser('~'))
+      txtFilter = _('Text files (*.txt)')
+      dialog.addFilter(txtFilter, ['*.txt'], None)
+      dialog.addFilter(_('All files'), ['*'], None)
+      dialog.set_icon_from_file(handlepaths.get_app_logo())
+      if dialog.show():
+        filename = dialog.filename
+        if dialog.lastFilter.get_name() == txtFilter and filename[-4:] != '.txt':
+          filename += '.txt'
+        dialog.destroy()
+    # Save selected filename
+    if filename:
       print 'saving text in %s' % filename
       file = None
       try:
@@ -317,15 +324,17 @@ class gespeakerUI(object):
       if file:
         file.close()
 
-  def on_imgmenuEditResetSettings_activate(self, widget, data=None):
+  def on_imgmenuEditResetSettings_activate(self, widget, confirm=True):
     "Restore default settings"
-    dialog = DialogYesNo(
-      message=_('Do you want to reset the default settings?'),
-      default_button=gtk.RESPONSE_NO
-    )
-    dialog.set_icon_from_file(handlepaths.get_app_logo())
-    dialog.show()
-    if dialog.responseIsYes():
+    if confirm:
+      dialog = DialogYesNo(
+        message=_('Do you want to reset the default settings?'),
+        default_button=gtk.RESPONSE_NO
+      )
+      dialog.set_icon_from_file(handlepaths.get_app_logo())
+      dialog.show()
+    # Reset if confirm was not requested or if the user gave response
+    if not confirm or dialog.responseIsYes():
       if self.defaultLanguageIndex:
         self.cboLanguages.set_active(self.defaultLanguageIndex)
       print 'restored default settings'
@@ -336,6 +345,19 @@ class gespeakerUI(object):
       Settings.load()
       self.loadSettings(True)
 
+
+  def on_imgmenuEditCut_activate(self, widget, data=None):
+    "Cut the selected text"
+    self.txvBuffer.cut_clipboard(self.clipboard, self.txvText.get_editable())
+
+  def on_imgmenuEditCopy_activate(self, widget, data=None):
+    "Copy the selected text"
+    self.txvBuffer.copy_clipboard(self.clipboard)
+    
+  def on_imgmenuEditPaste_activate(self, widget, data=None):
+    "Paste the clipboard in the buffer"
+    self.txvBuffer.paste_clipboard(self.clipboard, None, self.txvText.get_editable())
+    
   def on_imgmenuHelpAbout_activate(self, widget, data=None):
     "Show the about dialog"
     print 'show about dialog'
@@ -344,11 +366,11 @@ class gespeakerUI(object):
       version=handlepaths.APP_VERSION,
       comment=_('A GTK frontend for espeak'),
       copyright='Copyright 2009 Fabio Castelli',
-      license=open(handlepaths.getPath('doc', 'copyright'), 'r').read(), 
+      license=handlepaths.read_text_file('doc', 'copyright'), 
       website='http://code.google.com/p/gespeaker/',
       website_label='Gespeaker',
       authors=['Fabio Castelli <muflone@vbsimple.net>'],
-      translation=_('translation'), 
+      translation=handlepaths.read_text_file('doc', 'translators'),
       logo=handlepaths.get_app_logo(),
       icon=handlepaths.get_app_logo()
     )
@@ -361,10 +383,20 @@ class gespeakerUI(object):
     "Press button to pause or continue"
     self.btnPause.set_active(not self.btnPause.get_active())
 
-  def on_imgmenuEditRec_activate(self, widget, data=None):
+  def on_imgmenuFileRec_activate(self, widget, filename=None):
     "Press button to record or disable recording"
-    self.tlbRecord.set_active(not self.tlbRecord.get_active())
-    
+    if filename:
+      if not self.tlbRecord.get_active():
+        # Workaround to press the button without showing the dialog
+        self.recordToFileRequested = True
+        self.tlbRecord.set_active(True)
+        self.recordToFileRequested = False
+        self.set_record(filename)
+      else:
+        return 'recording was already active, no action'
+    else:
+      self.tlbRecord.set_active(not self.tlbRecord.get_active())
+
   def checkIfPlaying(self):
     "Check if a process is still running"
     if self.espeak.isPlaying():
@@ -412,8 +444,8 @@ class gespeakerUI(object):
       tmpFile.write(text)
       tmpFile.close()
       # Replace espeak's arguments with dialog values
-      language = self.listLanguages[self.cboLanguages.get_active()][1]
-      isMbrola = self.listLanguages[self.cboLanguages.get_active()][2]
+      language = self.listLanguages[self.cboLanguages.get_active()][COL_NAME]
+      isMbrola = self.listLanguages[self.cboLanguages.get_active()][COL_MBROLA]
       # Apply variant and voice to non-mbrola voices
       if not isMbrola:
         # Choose voice variant
@@ -437,7 +469,7 @@ class gespeakerUI(object):
       self.imgmenuEditPlay.set_sensitive(False)
       self.imgmenuEditStop.set_sensitive(True)
       self.imgmenuEditPause.set_sensitive(True)
-      self.imgmenuEditRec.set_sensitive(False)
+      self.imgmenuFileRec.set_sensitive(False)
       self.btnPause.set_sensitive(True)
       self.btnPlayStop.set_label('gtk-media-stop')
       self.tlbRecord.set_sensitive(False)
@@ -445,8 +477,9 @@ class gespeakerUI(object):
         self.espeak.play(cmd, self.cmdPlayer, self.recordToFile)
       else:
         args = {
-          '%l': '%s/%s' % (
+          '%l': '%s/%s/%s' % (
             Settings.get('VoicesmbPath'), 
+            language.replace('mb-', '', 1),
             language.replace('mb-', '', 1))
         }
         cmdMbrola = [Settings.cmdMbrola] + [
@@ -462,7 +495,7 @@ class gespeakerUI(object):
     self.imgmenuEditPlay.set_sensitive(True)
     self.imgmenuEditStop.set_sensitive(False)
     self.imgmenuEditPause.set_sensitive(False)
-    self.imgmenuEditRec.set_sensitive(True)
+    self.imgmenuFileRec.set_sensitive(True)
     self.btnPause.set_sensitive(False)
     self.btnPlayStop.set_label('gtk-media-play')
     if Settings.get('SingleRecord'):
@@ -477,6 +510,9 @@ class gespeakerUI(object):
     self.loadSettings(False)
 
   def on_tlbRecord_toggled(self, widget, data=None):
+    # Workaround to avoid dialog show if the record was requested through dbus
+    if self.recordToFileRequested:
+      return
     self.recordToFile = None
     if self.tlbRecord.get_active():
       dialog = DialogFileSave(
@@ -489,16 +525,24 @@ class gespeakerUI(object):
         filename = dialog.filename
         if filename[-4:] != '.wav':
           filename += '.wav'
-        print 'record to %s' % filename
-        self.recordToFile = filename
-        self.stbStatus.push(self.statusContextId, 
-          _('Recording audio track to: %s' % self.recordToFile))
+        self.set_record(filename)
       else:
         self.tlbRecord.set_active(False)
       dialog.destroy()
     else:
       self.stbStatus.pop(self.statusContextId)
 
+  def set_record(self, filename):
+    "Set record filename"
+    print 'record to %s' % filename
+    self.recordToFile = filename
+    self.stbStatus.push(self.statusContextId, 
+      _('Recording audio track to: %s' % self.recordToFile))
+    
+  def disable_record(self):
+    "Disable recording"
+    self.tlbRecord.set_active(False)
+    
   def on_radioVoice_toggled(self, widget, data=None):
     "Assign variants after voice type change"
     if widget.get_active():
@@ -510,7 +554,7 @@ class gespeakerUI(object):
 
   def on_cboLanguages_changed(self, widget, data=None):
     "Check if a mbrola voice has been selected"
-    status = self.listLanguages[self.cboLanguages.get_active()][2]
+    status = self.listLanguages[self.cboLanguages.get_active()][COL_MBROLA]
     self.imgMbrola.set_from_stock(size=gtk.ICON_SIZE_BUTTON,
       stock_id=status and gtk.STOCK_YES or gtk.STOCK_NO)
     self.lblVoice.set_sensitive(not status)
@@ -518,3 +562,76 @@ class gespeakerUI(object):
     self.radioVoiceFemale.set_sensitive(not status)
     self.lblVariants.set_sensitive(not status)
     self.cboVariants.set_sensitive(not status)
+
+  def set_text(self, text, insert_type=0):
+    "Set buffer text"
+    if insert_type == 0:
+      # Replace previous text
+      self.txvBuffer.set_text(text)
+    elif insert_type == 1:
+      # Insert at cursor
+      self.txvBuffer.insert_at_cursor(text)
+    elif insert_type == 2:
+      # Insert at the begin
+      self.txvBuffer.insert(self.txvBuffer.get_start_iter(), text)
+    elif insert_type == 3:
+      # Insert at the end
+      self.txvBuffer.insert(self.txvBuffer.get_end_iter(), text)
+
+  def set_position(self, position):
+    "Set window position"
+    position = position.split('x', 1)
+    return self.winMain.move(int(position[0]), int(position[1]))
+
+  def set_size(self, size):
+    "Set window size"
+    size = size.split('x', 1)
+    return self.winMain.resize(int(size[0]), int(size[1]))
+
+  def set_window(self, action, data=None):
+    "Execute window action"
+    return_value = None
+    if action=='hide':
+      return_value = self.winMain.hide()
+      plugins.signal_proxy('on_hidden')
+    elif action=='show':
+      return_value = self.winMain.show()
+      plugins.signal_proxy('on_shown')
+    elif action=='minimize':
+      return_value = self.winMain.iconify()
+    elif action=='unminimize':
+      return_value = self.winMain.deiconify()
+    elif action=='maximize':
+      return_value = self.winMain.maximize()
+    elif action=='unmaximize':
+      return_value = self.winMain.unmaximize()
+    elif action=='fullscreen':
+      return_value = self.winMain.fullscreen()
+    elif action=='unfullscreen':
+      return_value = self.winMain.unfullscreen()
+    elif action=='stick':
+      return_value = self.winMain.stick()
+    elif action=='unstick':
+      return_value = self.winMain.unstick()
+    elif action=='active':
+      return_value = self.winMain.is_active()
+    elif action=='activate':
+      return_value = self.winMain.present()
+    elif action=='get-opacity':
+      return_value = int(self.winMain.get_opacity() * 100)
+    elif action=='set-opacity':
+      return_value = self.winMain.set_opacity(0.01 * int(data))
+    elif action=='set-keep-above':
+      return_value = self.winMain.set_keep_above(True)
+    elif action=='unset-keep-above':
+      return_value = self.winMain.set_keep_above(False)
+    elif action=='set-keep-below':
+      return_value = self.winMain.set_keep_below(True)
+    elif action=='unset-keep-below':
+      return_value = self.winMain.set_keep_below(False)
+    elif action=='get-size':
+      return_value = 'x'.join(str(i) for i in list(self.winMain.get_size()))
+    elif action=='get-position':
+      return_value = 'x'.join(str(i) for i in list(self.winMain.get_position()))
+
+    return str(return_value)
